#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <signal.h>
#include <float.h>

#include "execute.h"
#include "lock.h"
#include "Light.h"
#include "LightSimplex.h"

static char * build_debug()
{
  static char dbg[MAX_FVAL];
  int         pos = 0;

  memset(dbg, '\0', MAX_FVAL);

  if (Flags.debug[WARN]     == 1) { dbg[pos] = 'w'; ++pos; }
  if (Flags.debug[VERBOSE]  == 1) { dbg[pos] = 'v'; ++pos; }
  if (Flags.debug[BUSY]     == 1) { dbg[pos] = 'b'; ++pos; }
  if (Flags.debug[TERSE]    == 1) { dbg[pos] = 't'; ++pos; }
  if (Flags.debug[GEO]      == 1) { dbg[pos] = 'g'; ++pos; }
  if (Flags.debug[INFILE]   == 1) { dbg[pos] = 'i'; ++pos; }
  if (Flags.debug[MINFIND]  == 1) { dbg[pos] = 'm'; ++pos; }
  if (Flags.debug[CHISQR]   == 1) { dbg[pos] = 'q'; ++pos; }
  if (Flags.debug[OPTIMIZE] == 1) { dbg[pos] = 'o'; ++pos; }
  if (Flags.debug[SURFACE]  == 1) { dbg[pos] = 's'; ++pos; }

  return dbg;
}
  
static char * build_cmdline(int full)
{
  int    i;
  char   temp[512];
  size_t size    = strlen(program_invocation_name) + 1;
  size_t len;
  char * cmdline = malloc(size);
  
  strncpy(cmdline, program_invocation_name, size);
  
  if (full)
    {
      snprintf(temp, sizeof(temp), " %s", build_debug());
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }
  else
    {
      /* Terse output, only final chi square */
      len = 1 + strlen(" -Dt");
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, " -Dt", len);
    }

  snprintf(temp, sizeof(temp), " -N%d", PhaseSteps);
  len = 1 + strlen(temp);
  size += len;
  cmdline = realloc(cmdline, size);
  strncat(cmdline, temp, len);
 
  if (Flags.blackbody == OFF) 
    {
      len = 1 + strlen(" -M");
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, " -M", len);
    }

  if (Binary[Primary].log_g > FLT_EPSILON)
    {
      snprintf(temp, sizeof(temp), " -OP %8.4g", Binary[Primary].log_g);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }
  if (Binary[Secondary].log_g > FLT_EPSILON)
    {
      snprintf(temp, sizeof(temp), " -OS %8.4g", Binary[Secondary].log_g);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }

  if (isAlbedoSet(Primary))
    {
      snprintf(temp, sizeof(temp), " -aP %8.4g", GetAlbedo(Primary));
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }
  if (isAlbedoSet(Secondary))
    {
      snprintf(temp, sizeof(temp), " -aS %8.4g", GetAlbedo(Secondary));
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }

  if (Flags.fractional == ON)
    {
      len = 1 + strlen(" -F");
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, " -F", len);
    }

  snprintf(temp, sizeof(temp), " -L%d", Flags.limb);
  len = 1 + strlen(temp);
  size += len;
  cmdline = realloc(cmdline, size);
  strncat(cmdline, temp, len);

  if (Flags.reflect > 0)
    {
      snprintf(temp, sizeof(temp), " -R%d", Flags.reflect);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }

  for (i = 0; i < NUM_MAG; ++i)
    {
      if (fabs(Orbit.Third[i]) >= FLT_EPSILON)
	{
	  snprintf(temp, sizeof(temp), " -3%c%f", Filters[i][0],Orbit.Third[i]);
	  len = 1 + strlen(temp);
	  size += len;
	  cmdline = realloc(cmdline, size);
	  strncat(cmdline, temp, len);
	}
    }

  /* -- Data Files 
   */
  if (FileList)
    {
      FileType  *current = FileList;

      while (current) 
	{
	  snprintf(temp, sizeof(temp), " -I %s", current->DataFile);
	  len = 1 + strlen(temp);
	  size += len;
	  cmdline = realloc(cmdline, size);
	  strncat(cmdline, temp, len);
	  current = current->nextFile;
	}
    }

  /* -- Spots 
   */
  for (i = 0; i < Flags.Spots1; ++i)
    {
      
      snprintf(temp, sizeof(temp), " -sP %8.4f %8.4f %8.4f %8.4f", 
	       Spot[Primary][i].longitude,
	       Spot[Primary][i].latitude,
	       Spot[Primary][i].radius,
	       Spot[Primary][i].dimfactor);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }


  for (i = 0; i < Flags.Spots2; ++i)
    {
      
      snprintf(temp, sizeof(temp), " -sS %8.4f %8.4f %8.4f %8.4f", 
	       Spot[Secondary][i].longitude,
	       Spot[Secondary][i].latitude,
	       Spot[Secondary][i].radius,
	       Spot[Secondary][i].dimfactor);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }


  /* -- Orbit 
   */
  snprintf(temp, sizeof(temp), " -tP %8.4g", Orbit.TruePeriod / 86400.);
  len = 1 + strlen(temp);
  size += len;
  cmdline = realloc(cmdline, size);
  strncat(cmdline, temp, len);
	  
  snprintf(temp, sizeof(temp), " -tM %8.4g", Orbit.TrueMass / 1.989E30);
  len = 1 + strlen(temp);
  size += len;
  cmdline = realloc(cmdline, size);
  strncat(cmdline, temp, len);

  if (Flags.elliptic == ON)
    {
      snprintf(temp, sizeof(temp), " -e %8.4g %8.4g", Orbit.Excentricity, Orbit.Omega);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }

  if (Flags.asynchron1 == ON)
    {
      snprintf(temp, sizeof(temp), " -fP %8.4g", Binary[Primary].Fratio);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
    }

  if (Flags.asynchron2 == ON)
    {
      snprintf(temp, sizeof(temp), " -fS %8.4g", Binary[Secondary].Fratio);
      len = 1 + strlen(temp);
      size += len;
      cmdline = realloc(cmdline, size);
      strncat(cmdline, temp, len);
     }

  snprintf(temp, sizeof(temp), " %8.4g %8.4g %8.4g %8.4g %8.4g %8.4g", 
	   Binary[Primary].Mq,
	   Orbit.Inclination / DTOR,
	   Binary[Primary].RocheFill,
	   Binary[Secondary].RocheFill,
	   Binary[Primary].Temperature,
	   Binary[Secondary].Temperature);
  len = 1 + strlen(temp);
  size += len;
  cmdline = realloc(cmdline, size);
  strncat(cmdline, temp, len);
 
  return cmdline;
}


/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Wrapper function for optional distributed execution
 @param     (double *) merit
 @return    (int)    status   The exit status
 @heading   Data Fitting
 ****************************************************************************/
#define N_MAXDIST 512
static double merits[N_MAXDIST];
static int    cookies[N_MAXDIST];
static int    result[N_MAXDIST];
static int    i_merit = 0;

#include "execute.h"

void DistReset()
{
  execute_cleanup();
  i_merit = 0;
  return;
}

int DistCheckStatus(int * count)
{
  int i;
  int status;
  double retval;
  int return_value = 0;
  
  for (i = 0; i < i_merit; ++i)
    {
      if (result[i] >= 1)
	{
	  status = check_result(cookies[i], &retval);
	  
	  if (status == 1 && retval != 0.0)
	    {
	      if (Flags.debug[PARALLEL] == ON)
		fprintf(stderr, "result #%02d ***** %f\n", i, retval);
	      result[i] = 0;
	      merits[i] = retval;
	      *count += 1;
	    }
	  else if (status == 1)
	    {
	      if (Flags.debug[PARALLEL] == ON)
		fprintf(stderr, "result #%02d BAD   %f\n", i, retval);
	      result[i] = -1;
	      merits[i] = PENALTY;
	      kill_this (cookies[i]);
	      return_value = -1;
	      *count += 1;
	    }
	  else if (status == -1)
	    {
	      if (Flags.debug[PARALLEL] == ON)
		fprintf(stderr, "result #%02d ERROR %f\n", i, retval);
	      
	      if (result[i] > 1)
		{
		  if (Flags.debug[PARALLEL] == ON)
		    fprintf(stderr, "result #%02d RE-EXECUTE\n", i);
		  if (re_execute(cookies[i], 1) != -2)
		    --result[i];
		}
	      else
		{
		  if (Flags.debug[PARALLEL] == ON)
		    fprintf(stderr, "result #%02d ERROR/BAD\n", i);
		  result[i] = -1;
		  merits[i] = PENALTY;
		  kill_this (cookies[i]);
		  return_value = -1;
		  *count += 1;
		}
	    }
	}
    }
  return return_value;
}

int DistExecLoop(double * merit)
{
  int status;
  static int initialized = 0;

  if (use_parallel())
    {
      char * cmdline = build_cmdline(0);

      *merit = PENALTY;

      if (!initialized)
	{
	  atexit(remove_all_locks);

	  if (Flags.debug[PARALLEL] == ON)
	    verbose_parallel(1);

	  ++initialized;
	}

      if (i_merit < N_MAXDIST)
	{
	  int eret;

	next_try:

	  eret = execute(cmdline, &(cookies[i_merit]), 1);

	  /* printf("-- executing |%s|\n", cmdline); */

	  if (0 == eret)
	    {
	      result[i_merit] = 5;
	      ++i_merit;
	    }
	  else if (-2 == eret)
	    {
	      /* need to wait until some slot is free */
	      int count = 0;
	      do {
		if (DistCheckStatus(&count) < 0)
		  {
		    WARNING(_("Executing subprocess failed!"));
		    free(cmdline);
		    return 8;
		  }
	      } while (count == 0);

	      goto next_try;
	    }
	  else
	    {
	      WARNING(_("Execute subprocess failed!"));
	      free(cmdline);
	      return 8;
	    }
	}
      else
	{
	  WARNING(_("Too many subprocesses!"));
	  free(cmdline);
	  return 8;
	}
      
      free(cmdline);
      return 0;
    }

  status = MainLoop(merit);
  merits[i_merit] = *merit;
  ++i_merit;

  return status;
}


int DistCollectMerits(double * merit /* must be <= [N_MAXDIST] */)
{
  int    count = 0;
  int    i;
  int    return_value = 0;

  if (!use_parallel())
    {
      for (i = 0; i < i_merit; ++i)
	{
	  merit[i] = merits[i];
	}
      i_merit = 0;
      return 0;
    }

  for (i = 0; i < i_merit; ++i)
    {
      if (result[i] < 1) 
	++count;
    }

  do {

    if (DistCheckStatus(&count) < 0)
      return 8;

    sleep(1);

  } while (count < i_merit);

  for (i = 0; i < i_merit; ++i)
    {
      merit[i] = merits[i];
    }
  i_merit = 0;

  execute_cleanup();

  return return_value;
}
