#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include <time.h>
#include <signal.h>

#include "lock.h"

#define P_CFG_FILE "parallel.cfg"

static int p_verbose = 0;

#include <stdarg.h>

static void p_report(char * format, ...)
{
  va_list ap;

  if (!p_verbose)
    return;

  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);

  return;
}

static char * get_basedir()
{
  size_t len = 256;
  char * p;
  char * buf = NULL;

  do {
    buf = realloc(buf, len);
    p = getcwd(buf, len);
    if (p == NULL)
      len += 256;
  } while (p == NULL);

  return p;
}

struct host_pq {
  char *  host;
  int      max;
  int      cur;
  time_t start; 
  struct host_pq * next;
};

static struct   host_pq * pq_list      = NULL;
static               int  pq_list_busy = 0;

static void pq_destroy()
{
  struct host_pq * this = pq_list;
  struct host_pq * old;

  while (this)
    {
      old  = this;
      this = this->next;

      free(old->host);
      free(old);
    }

  pq_list      = NULL;
  pq_list_busy = 0;

  return;
}

static void pq_add(const char * host, int max)
{
  struct host_pq * this = malloc(sizeof(struct host_pq));

  this->host  = strdup(host);
  this->max   = max;
  this->cur   = 0;
  this->start = 0;
  this->next = pq_list;

  pq_list = this;
  p_report("add host %s, max %d\n", this->host, this->max);
  return;
}

static char * pq_fetch()
{
  struct host_pq * this = pq_list;

  while (this)
    {
      if (this->cur < this->max)
	{
	  this->start = time(NULL);
	  ++(this->cur);
	  ++pq_list_busy;
	  p_report("inc host %s to %d (now %d busy)\n", 
		   this->host, this->cur, pq_list_busy);
	  return strdup(this->host);
	}
      this = this->next;
    }
  return NULL;
}

static void pq_avail(const char * host)
{
  struct host_pq * this = pq_list;

  while (this)
    {
      if (0 == strcmp(host, this->host))
	{
	  if (this->cur > 0) 
	    {
	      this->start = 0;
	      --(this->cur);
	      if (pq_list_busy > 0)
		--pq_list_busy;
	      p_report("dec host %s to %d (now %d busy)\n", 
		       host, this->cur, pq_list_busy);
	    }
	}
      this = this->next;
    }
  return;
}

int have_parallel_file ()
{
  FILE * fp = fopen(P_CFG_FILE, "r");
  if (fp)
    {
      fclose(fp);
      return 1;
    }
  return 0;
}

static char * get_free_host(char * basedir, int * errval)
{
  static int  initialized = 0;

  *errval = 0;

  p_report("get free host..\n");

  if ( (initialized > 0) && (pq_list_busy == 0) )
    {
      /* no active host, re-scan list */
      pq_destroy();
      initialized = 0;
    }

  if (0 == initialized)
    {
      FILE * fp = fopen(P_CFG_FILE, "r");

      (void) basedir;

      *errval = 0;

      if (fp)
	{
	  int ret;
	  
	  do {
	    char line[256];
	    char host[256];
	    int max;
	    
	    if (NULL == fgets(line, sizeof(line), fp))
	      break;
	    ret = sscanf(line, " HOST %255s %d", host, &max);
	    if (ret == 2)
	      p_report("get_free_host: %s max %d\n", host, max);
	    if (ret == 2 && max > 0)
	      {
		pq_add(host, max);
	      }
	  } while (1);

	  fclose(fp);
	  ++initialized;
	  p_report("host list initialized\n");
	}
      else
	{
	  *errval = 1;
	}
    }

  p_report("fetching free host..\n");
  return pq_fetch();
}

static int run_command(char * command, pid_t * cpid)
{
  char * arg0 = "sh";
  char * arg1 = "-c";
  char * arg2 = command;

  char *argc[] = { arg0, arg1, arg2, NULL };

  pid_t pid;

  p_report("run_command: %s\n", command);

  pid = fork();

  if (pid == 0)
    {
      execv("/bin/sh", argc);
      perror("run_command: execv");
      _exit(EXIT_FAILURE);
    }
  else if (pid == -1)
    {
      perror("run_command: fork");
      return -1;
    }

  *cpid = pid;
  p_report("run_command: pid = %d\n", *cpid);
  return 0;
}

struct cookie_store {
  char                * host;
  char                * file;
  char                * com;
  int                   cookie;
  int                   pid;
  struct cookie_store * next;
};

struct cookie_store * cookie_jar = NULL;

static void free_cookie(struct cookie_store * pre, struct cookie_store * cur)
{
  if (cur == cookie_jar)
    {
      cookie_jar = cur->next;
    }
  else
    {
      assert(pre != cur);
      pre->next = cur->next;
    }
  free(cur->host); free(cur->file); free(cur->com); free(cur);
  return;    
}

static void free_cookie_all()
{
  struct cookie_store * cur  = cookie_jar;
  struct cookie_store * next = cookie_jar;

  while (cur) 
    {
      next = cur->next;
      free(cur->host); free(cur->file); free(cur->com); free(cur);
      cur  = next;
    }
  cookie_jar = NULL;
  return;
}

static struct cookie_store * search_by_cookie (int cookie)
{
  struct cookie_store * cur = cookie_jar;

  while (cur)
    {
      if (cur->cookie == cookie)
	{
	  return cur;
	}
      cur = cur->next;
    }
  return NULL;
}

static char * print_full_command (char * free_host, char * command, 
			     char * result_file)
{
  size_t len = strlen(command) + 256;
  char * full_command = malloc(len);
  assert(full_command != NULL);
  
  snprintf(full_command, len,
	   "ssh -n -o BatchMode=yes %s '%s' > %s",
	   free_host, command, result_file);
  return full_command;
}


static int runit (char * command, 
		  char * result_file, char * free_host, int * cookie)
{
  pid_t cpid;

  char * full_command = print_full_command (free_host, command, 
					    result_file);

  /* p_report("runit: |%s|\n", full_command); */

  if (0 == run_command(full_command, &cpid))
    {
      struct cookie_store * cs = search_by_cookie(*cookie);

      if (cs == NULL)
	{
	  cs = malloc(sizeof(struct cookie_store));

	  cs->host   = strdup(free_host);
	  cs->file   = strdup(result_file);
	  cs->com    = strdup(command);
	  cs->cookie = (int) cpid;
	  cs->pid    = cpid;
	  *cookie    = (int) cpid;

	  cs->next   = cookie_jar;
	  cookie_jar = cs;
	}
      else
	{
	  free(cs->host);
	  cs->host   = strdup(free_host);
	  free(cs->file);
	  cs->file   = strdup(result_file);
	  /* 
	   * Don't fiddle with cs->com, because
	   * (a) it has not changed, and
	   * (b) worse, it is the actual function argument.
	   */
	  cs->pid    = cpid;
	}

      free(full_command);
      return 0;
    }

  free(full_command);
  return -1;
}

static int is_finished(pid_t pid, int * exitval)
{
  int stat_val;
  pid_t res;

  res = waitpid(pid, &stat_val, WNOHANG);

  if (res == pid)
    {
      if (WIFEXITED(stat_val))
	{
	  *exitval = WEXITSTATUS(stat_val);
	  return 0;
	}
      else
	{
	  *exitval = -1;
	  return 0;
	}
    }

  return -1;
}

static double fetch_result(char * file, int * errval)
{
  double retval = 0.0;
  FILE * fp = fopen(file, "r");

  if (fp)
    {
      if (1 != fscanf(fp, "%lf", &retval))
	{
	  if (ferror(fp))
	    perror("fetch_result: fscanf");
	  retval = 0.0;
	  *errval = -1;
	}
      else
	{
	  p_report("fetch_result: retval = %e\n", retval);
	  *errval = 0;
	}
      fclose (fp);
    }
  else
    {
      *errval = -1;
      perror("fetch_result: fopen");
    }

  if (*errval == 0)
    unlink(file);

  return retval;
}

void purge_cookie(int cookie)
{
  struct cookie_store * old = cookie_jar;
  struct cookie_store * new = cookie_jar;

  while (new)
    {
      if (new->cookie == cookie)
	{
	  free_cookie(old, new);
	  return;
	}
      old = new;
      new = new->next;
    }
  return;
}

void execute_cleanup()
{
  free_cookie_all();
  pq_destroy();
  return;
}

void kill_this(int cookie)
{
  struct cookie_store * old = cookie_jar;
  struct cookie_store * new = cookie_jar;

  p_report("kill_this: cookie %d -- enter --\n", cookie);

  while (new)
    {
      if (new->cookie == cookie)
	{
	  pid_t  pid = new->pid;
	  int    try = 3;
	  
	  p_report("kill_this: cookie %d host %s killing process %d\n", 
		   cookie, new->host, (int) pid);
	  kill(pid, SIGKILL);
	  do {
	    if (waitpid(pid, NULL, 0) > 0)
	      break;
	    --try;
	    sleep(1);
	  } while (try > 0);
	  pq_avail(new->host);
	  unlink(new->file);
	  free_cookie(old, new);
	  return;
	}
      old = new;
      new = new->next;
    }
  p_report("kill_this: cookie %d NOT FOUND\n", cookie);
  return;
}

/* return: 1 on finished, 0 on running, -1 on error
 */
int check_result(int cookie, double * retval)
{
  struct cookie_store * old = cookie_jar;
  struct cookie_store * new = cookie_jar;

  while (new)
    {
      if (new->cookie == cookie)
	{
	  int exitval;
	  pid_t  pid = new->pid;

	  if (0 == is_finished(pid, &exitval))
	    {
	      p_report("check_result: cookie %d finished\n", cookie);
	      pq_avail(new->host);

	      if (exitval == 0)
		{
		  p_report("check_result: cookie %d terminated\n", cookie);
		  *retval = fetch_result(new->file, &exitval);
		  if (exitval == 0)
		    {
		      free_cookie(old, new);
		      return 1;
		    }
		  else
		    {
		      return -1;
		    }
		}
	      else
		{
		  *retval = 0.0;
		  unlink(new->file);
		  return -1;
		}
	    }
	  return 0;
	}
      old = new;
      new = new->next;
    }
  return -1;
}

void verbose_parallel(int is_verbose)
{
  p_verbose = is_verbose;
}

int use_parallel()
{
  int res = access(P_CFG_FILE, R_OK);

  if (res == 0) /* File exists */
    return 1;

  return 0;
}

static char * choose_free_host(char * basedir, int tries)
{
  char * free_host = NULL;
  int    count = 0;
  do {
    int errval;
    free_host = get_free_host(basedir, &errval);
    if (errval)
      {
	p_report("get_free_host errval = %d\n", errval);
	return NULL;
      }
    ++count;
  } while (!free_host && count < tries);

  return free_host;
}

int execute(char * command, int * cookie, int tries)
{
  char * free_host = NULL;
  char * basedir;
  char * result_file_template = ".parallel_resultXXXXXX";
  char   result_file[256];
  int    result_fd;
  int    ret;

  *cookie = -1;

  strncpy(result_file, result_file_template, sizeof(result_file));
  result_file[sizeof(result_file)-1] = '\0';

  result_fd = mkstemp(result_file);
  if (result_fd < 0)
    {
      perror("execute: mkstemp");
      return -1;
    }
  close(result_fd);

  basedir = get_basedir();
  if (!basedir)
    return -1;

  free_host = choose_free_host(basedir, tries);
  if (!free_host)
    {
      p_report("no free host currently\n");
      return -2;
    }

  /* p_report("exec |%s|\n", command); */
  ret = runit (command, result_file, free_host, cookie);
  p_report("execute: cookie %d host %s returned %d\n", *cookie, free_host, ret);
  free(basedir);
  free(free_host);
  return ret;
}

int re_execute(int cookie, int tries)
{
  char * command;
  char * free_host = NULL;
  char * basedir;
  char * result_file_template = ".parallel_resultXXXXXX";
  char   result_file[256];
  int    result_fd;
  int    ret;
  struct cookie_store * cs = search_by_cookie(cookie);

  if (!cs)
    return -1;

  strncpy(result_file, result_file_template, sizeof(result_file));
  result_file[sizeof(result_file)-1] = '\0';

  result_fd = mkstemp(result_file);
  if (result_fd < 0)
    {
      perror("execute: mkstemp");
      return -1;
    }
  close(result_fd);

  basedir = get_basedir();
  if (!basedir)
    return -1;

  free_host = choose_free_host(basedir, tries);
  if (!free_host)
    return -2;

  command = cs->com;

  p_report("re-exec |%s|\n", command);
  ret = runit (command, result_file, free_host, &cookie);
  p_report("execute: cookie %d host %s returned %d\n", cookie, free_host, ret);
  free(basedir);
  free(free_host);
  return ret;
}
