#define _XOPEN_SOURCE 500
#include <sys/types.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>

struct lock_file {
  char * file;
  struct lock_file * next;
};

static struct lock_file * lf_list = NULL;

static void free_lockfile(struct lock_file * pre, struct lock_file * cur)
{
  if (cur == lf_list)
    {
      lf_list = cur->next;
    }
  else
    {
      assert(pre != cur);
      pre->next = cur->next;
    }
  free(cur->file); free(cur);
  return;    
}

static void free_this_lockfile(char * file)
{
  struct lock_file * pre = lf_list;
  struct lock_file * cur = lf_list;

  while (cur)
    {
      if (0 == strcmp(cur->file, file))
	{
	  free_lockfile(pre, cur);
	  return;
	}
      pre = cur;
      cur = cur->next;
    }
  return;
}

static void add_this_lockfile(char * file)
{
  struct lock_file * cur = malloc(sizeof(struct lock_file));
  
  if (cur)
    {
      size_t len = 1 + strlen(file);
      cur->file = malloc(len);
      strcpy(cur->file, file);
      /* cur->file = strdup(file); *//* osx doesn't have it */
      cur->next = lf_list;
      lf_list   = cur;
    }
  return;
}

void remove_all_locks()
{
  struct lock_file * cur = lf_list;

  while (cur)
    {
      unlink(cur->file);
      cur = cur->next;
    }
  return;
}
  

static int write_pid(int fd)
{
  pid_t  pid      = getpid();
  FILE * fp;

  fp = fdopen(fd, "w");
  if (!fp)
    {
      perror("write_pid: fdopen");
      return -1;
    }

  if (fprintf(fp, "%d", (int) pid) < 0)
    {
      perror("write_pid: fprintf");
      return -1;
    }

  if (0 != fclose(fp))
    {
      perror("write_pid: fclose");
      return -1;
    }
  return 0;
}

static int pid_valid(int pid)
{
  return kill(pid, 0);
}

static int test_lock(char * lockfile)
{
  int    fpid;
  int    pid  = (int) getpid();
  FILE * fp   = fopen(lockfile, "r");

  if (!fp)
    {
      perror("test_lock: fopen");
      return -1;
    }
  if (1 != fscanf(fp, "%d", &fpid))
    {
      perror("test_lock: fscanf");
      fclose(fp);
      return -1;
    }
  if (pid != fpid)
    {
      fclose(fp);
      return pid_valid(fpid);
    }

  fclose(fp);
  return 0;
}

static int try_lock(char * template, char * lockfile)
{
  int ret = 0;
  int ttt = 5;

  do {
    ret = link(template, lockfile);
    if (ret == -1)
      {
	if (errno == EEXIST)
	  {
	    sleep(ttt); 
	    ttt+= 5;
	  }
	else
	  {
	    perror("try_lock: link");
	    return -1;
	  }
      }
  } while (ret < 0 && ttt < 65);

  return ret;
}

static int create_lockfile(char * lockfile)
{
  int    fd;
  int    ret;
  char * template = malloc(strlen(lockfile) + 7);

  assert(template != NULL);

  strcpy(template, lockfile);
  strcat(template, "XXXXXX");
			   
  fd = mkstemp(template);
  if (fd == -1)
    {
      free(template);
      perror("create_lockfile: mkstemp");
      return -1;
    }

  if (write_pid(fd) == -1)
    {
      unlink(template);    
      free(template);
      return -1;
    }

  ret = try_lock(template, lockfile);
  if (ret < 0)
    {
      unlink(template);
      free(template);
      return -1;
    }
 
  unlink(template);
  free(template);
  return 0;
}

int acquire_lock(char * file)
{
  char * lockfile = malloc(strlen(file) + 6);

  assert(lockfile != NULL);

  strcpy(lockfile, file);
  strcat(lockfile, ".lock");

  add_this_lockfile(lockfile);
  if (create_lockfile(lockfile) < 0)
    {
      free_this_lockfile(lockfile);
      free(lockfile);
      return -1;
    }

  
  free(lockfile);
  return 0;
}

int valid_lock(char * file)
{
  struct stat buf;
  int ret;
  char * lockfile = malloc(strlen(file) + 6);

  assert(lockfile != NULL);

  strcpy(lockfile, file);
  strcat(lockfile, ".lock");

  ret = stat(lockfile, &buf);
  if (ret < 0)
    {
      free(lockfile);
      return -1;
    }

  ret = test_lock(lockfile);
  return ret;
}

int remove_lock(char * file)
{
  int ret;
  char * lockfile = malloc(strlen(file) + 6);

  assert(lockfile != NULL);

  strcpy(lockfile, file);
  strcat(lockfile, ".lock");

  ret = unlink(lockfile);

  if (ret < 0 && errno != ENOENT)
    {
      perror("remove_lock: unlink");
      ret = -1;
    }
  else
    {
      free_this_lockfile(lockfile);
      ret = 0;
    }

  free(lockfile);
  return ret;
}


